@@ -1,12 +1,13 @@
 #include <RcppArmadillo.h>
 #include <cmath>
+#include <boost/math/special_functions/digamma.hpp>
+#include <boost/math/special_functions/trigamma.hpp>
 #ifdef _OPENMP
 # include <omp.h>
 #endif
 using namespace Rcpp;
 
-// Calculating sum of logs of factorials 
-
+// Calculating sum of logs of factorials, this is used for poisson likelihood
 double LogFact(const arma::vec* y){
   double sum = 0;
   double Max = max(*y);
@@ -24,7 +25,6 @@ double LogFact(const arma::vec* y){
 }
 
 // Checking bounds
-
 void CheckBounds(arma::vec* mu, std::string Dist){
   if(Dist == "binomial"){
     mu->transform([](double val){
@@ -33,7 +33,7 @@ void CheckBounds(arma::vec* mu, std::string Dist){
       return(val);
     });
   }
-  else if(Dist == "poisson"){
+  else if(Dist == "poisson" || Dist == "gamma"){
     mu->transform([](double val){
       if(val <= 0){val = FLT_EPSILON;}
       return(val);
@@ -42,17 +42,17 @@ void CheckBounds(arma::vec* mu, std::string Dist){
 }
 
 // Defining Link functions
-
 arma::vec LinkCpp(const arma::mat* X, arma::vec* beta, const arma::vec* Offset, 
                   std::string Link, std::string Dist){
   
+  // Calculating linear predictors
   arma::vec XBeta = (*X * *beta) + *Offset;
   arma::vec mu(XBeta.n_elem);
   
   if(Link == "log"){
 #pragma omp parallel for
     for(unsigned int i = 0; i < Offset->n_elem; i++){
-       mu.at(i) = exp(XBeta.at(i));
+      mu.at(i) = exp(XBeta.at(i));
     }
   }
   else if(Link == "logit"){
@@ -71,13 +71,13 @@ arma::vec LinkCpp(const arma::mat* X, arma::vec* beta, const arma::vec* Offset,
     }
   }
   else if(Link == "inverse"){
-    mu = 1 / (XBeta);
+    mu = -1 / (XBeta);
   }
   else if(Link == "identity"){
     mu = XBeta;
   }
-  else{
-    mu = sqrt(XBeta);
+  else if(Link == "sqrt"){
+    mu = pow(XBeta, 2);
   }
   
   CheckBounds(&mu, Dist);
@@ -85,8 +85,7 @@ arma::vec LinkCpp(const arma::mat* X, arma::vec* beta, const arma::vec* Offset,
   return(mu);
 }
 
-// Defining Derivative functions
-
+// Defining Derivative functions for each link function
 arma::vec DerivativeCpp(const arma::mat* X, arma::vec* beta, const arma::vec* Offset,
                         arma::vec* mu, std::string Link, std::string Dist){
   
@@ -111,20 +110,18 @@ arma::vec DerivativeCpp(const arma::mat* X, arma::vec* beta, const arma::vec* Of
     }
   }
   else if(Link == "inverse"){
-    Deriv = -pow(*mu, 2);
+    Deriv = pow(*mu, 2);
   }
   else if(Link == "identity"){
     Deriv.fill(1);
-  }
-  else{
-    Deriv = 1 / (2 * *mu);
+  }else if(Link == "sqrt"){
+    Deriv = 2 * sqrt(*mu);
   }
   
   return(Deriv);
 }
 
-// Defining Link functions
-
+// Defining Variance functions for each family
 arma::vec Variance(arma::vec* mu, std::string Dist){
   
   arma::vec Var(mu->n_elem);
@@ -132,12 +129,6 @@ arma::vec Variance(arma::vec* mu, std::string Dist){
   if(Dist == "poisson"){
     Var = *mu; 
   }
-  else if(Dist == "Negative binomial"){
-#pragma omp parallel for
-    for(unsigned int i = 0; i < mu->n_elem; i++){
-      Var.at(i) = mu->at(i) * pow(mu->at(i), 2);
-    }
-  }
   else if(Dist == "binomial"){
 #pragma omp parallel for
     for(unsigned int i = 0; i < mu->n_elem; i++){
@@ -160,8 +151,7 @@ arma::vec Variance(arma::vec* mu, std::string Dist){
   
 }
 
-// Defining log likelihood for pointer armadillo vector beta
-
+// Defining log likelihood function
 double LogLikelihoodCpp(const arma::mat* X, const arma::vec* Y, 
                         arma::vec* mu, std::string Dist){
   
@@ -179,27 +169,26 @@ double LogLikelihoodCpp(const arma::mat* X, const arma::vec* Y,
       double theta = mu->at(i) / (1 - mu->at(i));
       LogLik += -Y->at(i) * log(theta) + log1p(theta);
     }
- }else if(Dist == "gamma"){
+  }else if(Dist == "gamma"){
     arma::vec theta = -1 / *mu;
-    LogLik = arma::dot(*Y, theta) - arma::accu(log(-theta));
- }else{
+    LogLik = -arma::dot(*Y, theta) - arma::accu(log(-theta));
+  }else{
 #pragma omp parallel for reduction(+:LogLik)
-   for(unsigned int i = 0; i < Y->n_elem; i++){
-     LogLik += pow(Y->at(i) - mu->at(i), 2) /2;
+    for(unsigned int i = 0; i < Y->n_elem; i++){
+      LogLik += pow(Y->at(i) - mu->at(i), 2) /2;
+    }
   }
- }
   return(LogLik);
 }
 
 // Defining log likelihood for saturated model
-
 double LogLikelihoodSat(const arma::mat* X, const arma::vec* Y, std::string Dist){
   
   double LogLik = 0;
   
   if(Dist == "poisson"){
     for(unsigned int i = 0; i< Y->n_elem;i++){
-      if(Y->at(i) !=0){
+      if(Y->at(i) != 0){
         LogLik += Y->at(i) * (log(Y->at(i)) - 1);
       }
     }
@@ -208,7 +197,7 @@ double LogLikelihoodSat(const arma::mat* X, const arma::vec* Y, std::string Dist
     LogLik = 0;
   }else if(Dist == "gamma"){
     arma::vec theta = -1 / *Y;
-    LogLik = -arma::dot(*Y, theta) + arma::accu(log(-theta));
+    LogLik = arma::dot(*Y, theta) + arma::accu(log(-theta));
   }else{
     LogLik = 0;
   }
@@ -216,16 +205,19 @@ double LogLikelihoodSat(const arma::mat* X, const arma::vec* Y, std::string Dist
   return(LogLik);
 }
 
-// Defining score with pointers
-
+// Defining score function
 arma::vec ScoreCpp(const arma::mat* X, const arma::vec* Y, arma::vec* Deriv,
                    arma::vec* Var, arma::vec* mu){
   
+  // Initializing vector to store results
   arma::vec FinalVec(X->n_cols);
+  
+  //Calculating weights and difference in observed and expected
   arma::vec w = *Deriv / *Var;
   arma::vec diff = *Y - *mu;
   w.replace(arma::datum::nan, 0);
   
+  // Calculating score
 #pragma omp parallel for
   for(unsigned int i = 0; i < X->n_cols; i++){
     
@@ -235,21 +227,24 @@ arma::vec ScoreCpp(const arma::mat* X, const arma::vec* Y, arma::vec* Deriv,
   return FinalVec;
 }
 
-// Defining fisher information with pointer
-
+// Defining fisher information function
 arma::mat FisherInfoCpp(const arma::mat* X, arma::vec* Deriv, 
                         arma::vec* Var){
   
+  // Initializing matrix to store results
   arma::mat FinalMat(X->n_cols, X->n_cols);
+  
+  // Calculating weight vector, this is the diagonal of the W matrix
   arma::vec w = pow(*Deriv, 2) / *Var;
   w.replace(arma::datum::nan, 0);
   checkUserInterrupt();
   
+  // Calculating fisher information
 #pragma omp parallel for schedule(dynamic, 1)
   for(unsigned int i = 0; i < X->n_cols; i++){
     
     FinalMat(i, i) = arma::dot((X->col(i) % w), X->col(i));
-
+    
     for(unsigned int j = i + 1; j < X->n_cols; j++){
       
       FinalMat(i, j) = arma::dot((X->col(j) % w), X->col(i));
@@ -261,10 +256,9 @@ arma::mat FisherInfoCpp(const arma::mat* X, arma::vec* Deriv,
   return FinalMat;
 }
 
-// Defining fisher information with pointer
-
+// Defining fisher information test function
 arma::mat FisherInfoCppNew(const arma::mat* X, arma::vec* Deriv, 
-                        arma::vec* Var){
+                           arma::vec* Var){
   
   arma::mat FinalMat(X->n_cols, X->n_cols);
   arma::vec w = pow(*Deriv, 2) / *Var;
@@ -290,8 +284,7 @@ arma::mat FisherInfoCppNew(const arma::mat* X, arma::vec* Deriv,
   return FinalMat;
 }
 
-// Creating LBFGS helper function
-
+// LBFGS helper function
 arma::vec LBFGSHelperCpp(arma::vec* g1, arma::mat* s, arma::mat* y, 
                          int* k, int* m, 
                          arma::vec* r, arma::vec* alpha, const arma::mat* Info){
@@ -314,17 +307,18 @@ arma::vec LBFGSHelperCpp(arma::vec* g1, arma::mat* s, arma::mat* y,
     return *r;
   }
   
+  // Returns this if no gradients have been evaluated yet
   return *Info * *g1;
 }
 
-// Creating LBFGS for logistic regression
+// LBFGS
 int LBFGSGLMCpp(arma::vec* beta, const arma::mat* X, 
-                   const arma::vec* Y, const arma::vec* Offset,
-                   std::string Link, std::string Dist, 
-                   double tol, int maxit, int m = 5, 
-                   double C1 = pow(10, -4)){
+                const arma::vec* Y, const arma::vec* Offset,
+                std::string Link, std::string Dist, 
+                double tol, int maxit, int m = 5, 
+                double C1 = pow(10, -4)){
   
-  int k = 0;
+  // Initializing vectors and matrices 
   arma::vec mu = LinkCpp(X, beta, Offset, Link, Dist);
   arma::vec Deriv = DerivativeCpp(X, beta, Offset, &mu, Link, Dist);
   arma::vec Var = Variance(&mu, Dist);
@@ -340,11 +334,15 @@ int LBFGSGLMCpp(arma::vec* beta, const arma::mat* X,
     warning("Fisher information not invertible");
     return(-2);
   }
+  
+  // Initializing int and doubles
+  int k = 0;
   double f0;
   double f1 = LogLikelihoodCpp(X, Y, &mu, Dist);
   double t;
   double alpha;
   
+  // Fitting the model
   while(arma::norm(g1) > tol){
     checkUserInterrupt();
     if(k == maxit){ 
@@ -355,6 +353,8 @@ int LBFGSGLMCpp(arma::vec* beta, const arma::mat* X,
     alpha = 1;
     g0 = g1;
     f0 = f1;
+    
+    // Calculating p (search direction) based on L-BFGS approximation to inverse info
     p = -LBFGSHelperCpp(&g1, &s, &y, &k, &m, &r, &alphavec, &Info);
     t = -C1 * arma::dot(g0, p);
     *beta += alpha * p;
@@ -362,7 +362,6 @@ int LBFGSGLMCpp(arma::vec* beta, const arma::mat* X,
     f1 = LogLikelihoodCpp(X, Y, &mu, Dist);
     
     // Finding alpha with backtracking linesearch using Armijo-Goldstein condition
-    
     while((f0 < f1 + alpha * t) && (alpha > C1)){
       alpha /= 2;
       *beta -= alpha * p;
@@ -370,8 +369,9 @@ int LBFGSGLMCpp(arma::vec* beta, const arma::mat* X,
       f1 = LogLikelihoodCpp(X, Y, &mu, Dist);
     }
     
+    // Checking for convergence or nan/inf
     if(abs(f1 -  f0) < tol || all(abs(alpha * p) < tol)){
-      if(std::isinf(f1)){
+      if(std::isinf(f1) || beta->has_nan()){
         warning("LBFGS failed to converge");
         k = -2;
       }
@@ -389,14 +389,13 @@ int LBFGSGLMCpp(arma::vec* beta, const arma::mat* X,
 }
 
 
-// Creating BFGS for logistic regression
-
+// BFGS
 int BFGSGLMCpp(arma::vec* beta, const arma::mat* X, 
-                  const arma::vec* Y, const arma::vec* Offset,
-                  std::string Link, std::string Dist,
-                  double tol, int maxit, double C1 = pow(10, -4)){
+               const arma::vec* Y, const arma::vec* Offset,
+               std::string Link, std::string Dist,
+               double tol, int maxit, double C1 = pow(10, -4)){
   
-  int k = 0;
+  // Initializing vectors and matrices
   arma::vec mu = LinkCpp(X, beta, Offset, Link, Dist);
   arma::vec Deriv = DerivativeCpp(X, beta, Offset, &mu, Link, Dist);
   arma::vec Var = Variance(&mu, Dist);
@@ -410,12 +409,16 @@ int BFGSGLMCpp(arma::vec* beta, const arma::mat* X,
     warning("Fisher information not invertible");
     return(-2);
   }
+  
+  // Initializing int and doubles
+  int k = 0;
   double f0;
   double f1 = LogLikelihoodCpp(X, Y, &mu, Dist);
   double rho;
   double alpha;
   double t;
   
+  // Fitting the model
   while(arma::norm(g1) > tol){
     checkUserInterrupt();
     if(k == maxit){ 
@@ -443,8 +446,9 @@ int BFGSGLMCpp(arma::vec* beta, const arma::mat* X,
     
     k++;
     
+    // Checking for convergence or nan/inf
     if(abs(f1 -  f0) < tol || all(abs(alpha * p) < tol)){
-      if(std::isinf(f1)){
+      if(std::isinf(f1)|| beta->has_nan()){
         warning("BFGS failed to converge");
         k = -1;
       }
@@ -458,7 +462,6 @@ int BFGSGLMCpp(arma::vec* beta, const arma::mat* X,
     rho = 1/arma::dot(s, y);
     
     // Calculating next approximate inverse hessian
-    
     H1 = (arma::diagmat(arma::ones(beta->n_elem)) - rho * s * y.t()) * H1 * 
       (arma::diagmat(arma::ones(beta->n_elem)) - rho * y * s.t()) + rho * s * s.t();
   }
@@ -466,26 +469,30 @@ int BFGSGLMCpp(arma::vec* beta, const arma::mat* X,
 }
 
 
-// Creating Fisher Scoring for logistic regression
+// Fisher's scoring
 
 int FisherScoringGLMCpp(arma::vec* beta, const arma::mat* X, 
-                               const arma::vec* Y, const arma::vec* Offset,
-                               std::string Link, std::string Dist,
-                               double tol, int maxit,  
-                               double C1 = pow(10, -4)){
+                        const arma::vec* Y, const arma::vec* Offset,
+                        std::string Link, std::string Dist,
+                        double tol, int maxit,  
+                        double C1 = pow(10, -4)){
   
-  int k = 0;
+  // Initializing vector and matrices
   arma::vec mu = LinkCpp(X, beta, Offset, Link, Dist);
   arma::vec Deriv = DerivativeCpp(X, beta, Offset, &mu, Link, Dist);
   arma::vec Var = Variance(&mu, Dist);
   arma::vec g1 = ScoreCpp(X, Y, &Deriv, &Var, &mu);
   arma::vec p(beta->n_elem);
   arma::mat H1 = FisherInfoCpp(X, &Deriv, &Var);
+  
+  // Initializing int and doubles
+  int k = 0;
   double f0;
   double f1 = LogLikelihoodCpp(X, Y, &mu, Dist);
   double alpha;
   double t;
   
+  // Fitting the model
   while(arma::norm(g1) > tol){
     checkUserInterrupt();
     alpha = 1;
@@ -516,8 +523,9 @@ int FisherScoringGLMCpp(arma::vec* beta, const arma::mat* X,
     }
     k++;
     
+    //Checking for convergence or nan/inf
     if(abs(f1 -  f0) < tol || all(abs(alpha * p) < tol)){
-      if(std::isinf(f1)){
+      if(std::isinf(f1)|| beta->has_nan()){
         warning("Fisher Scoring failed to converge");
         k = -1;
       }
@@ -531,24 +539,145 @@ int FisherScoringGLMCpp(arma::vec* beta, const arma::mat* X,
   return(k);
 }
 
+int LinRegCpp(arma::vec* beta, const arma::mat* x, const arma::mat* y,
+              const arma::vec* offset, arma::vec* SE1){
+  
+  arma::mat FinalMat(x->n_cols, x->n_cols);
+  
+  // Finding X'X
+#pragma omp parallel for schedule(dynamic, 1)
+  for(unsigned int i = 0; i < x->n_cols; i++){
+    
+    FinalMat(i, i) = arma::dot(x->col(i), x->col(i));
+    
+    for(unsigned int j = i + 1; j < x->n_cols; j++){
+      
+      FinalMat(i, j) = arma::dot(x->col(j), x->col(i));
+      FinalMat(j, i) = FinalMat(i, j);
+    } 
+  }
+  
+  // calculating inverse of X'X
+  arma::mat InvXX(x->n_cols, x->n_cols);
+  if(!arma::inv_sympd(InvXX, FinalMat)){
+    warning("Fisher info not invertible");
+    return(-2);
+  }
+  
+  // Calculating beta, dispersion parameter, and beta variances
+  *beta = InvXX * x->t() * (*y - *offset);
+  *SE1 = arma::diagvec(InvXX);
+  return(1);
+} 
+
+int LinRegCppShort(arma::vec* beta, const arma::mat* x, const arma::mat* y,
+                   const arma::vec* offset){
+  
+  arma::mat FinalMat(x->n_cols, x->n_cols);
+  
+  // Finding X'X
+#pragma omp parallel for schedule(dynamic, 1)
+  for(unsigned int i = 0; i < x->n_cols; i++){
+    
+    FinalMat(i, i) = arma::dot(x->col(i), x->col(i));
+    
+    for(unsigned int j = i + 1; j < x->n_cols; j++){
+      
+      FinalMat(i, j) = arma::dot(x->col(j), x->col(i));
+      FinalMat(j, i) = FinalMat(i, j);
+    } 
+  }
+  
+  // calculating inverse of X'X
+  arma::mat InvXX(x->n_cols, x->n_cols);
+  if(!arma::inv_sympd(InvXX, FinalMat)){
+    warning("Fisher info not invertible");
+    return(-2);
+  }
+  
+  // Calculating beta, dispersion parameter, and beta variances
+  *beta = InvXX * x->t() * (*y - *offset);
+  return(1);
+}
+
+double GetDispersion(const arma::mat* X, const arma::vec* Y, 
+                     arma::vec* mu, double LogLik, std::string Dist, 
+                     double tol){
+  double dispersion = 1;
+  if(Dist == "gaussian"){
+    dispersion = arma::accu(pow(*Y - *mu, 2)) / (X->n_rows);
+  }else if(Dist == "gamma"){
+    unsigned int it = 0;
+    double dispersion2 = 0;
+    double fixed = LogLik + arma::accu(log(*Y)) + X->n_rows;
+    
+    // Initializing score and info
+    double score = fixed + X->n_rows * (log(dispersion) - boost::math::digamma(dispersion)); 
+    
+    double info = X->n_rows * (-1 / dispersion + boost::math::trigamma(dispersion));
+    
+    while(abs(score) > tol && abs(dispersion - dispersion2) > tol && it < 25){
+      dispersion += score / info;
+      score = fixed + X->n_rows * (log(dispersion) - boost::math::digamma(dispersion));
+      info = X->n_rows * (-1 / dispersion + boost::math::trigamma(dispersion));
+      it++;
+    }
+    dispersion = 1 / dispersion;
+  }
+  return(dispersion);
+}
+
+// Gets initial values for gamma and gaussian regression with log/inverse/sqrt link 
+// and gamma regression with identity link with transformed y linear regression
+void getInit(arma::vec* beta, const arma::mat* X, const arma::vec* Y, 
+             const arma::vec* Offset, std::string Dist, std::string Link){
+  
+  if(Link == "log" && (Dist == "gamma" || Dist == "gaussian")){
+    const arma::vec NewY = log(*Y);
+    LinRegCppShort(beta, X, &NewY, Offset);
+  }else if(Link == "inverse" && (Dist == "gamma" || Dist == "gaussian")){
+    const arma::vec NewY = -1 / (*Y);
+    LinRegCppShort(beta, X, &NewY, Offset);
+  }else if(Link == "sqrt" && (Dist == "gamma" || Dist == "gaussian")){
+    const arma::vec NewY = sqrt(*Y);
+    LinRegCppShort(beta, X, &NewY, Offset);
+  }else if(Link == "identity" && (Dist == "gamma")){
+    LinRegCppShort(beta, X, Y, Offset);
+  }
+}
 
 // [[Rcpp::export]]
 List BranchGLMfit(NumericMatrix x, NumericVector y, NumericVector offset,
-                     std::string method,  unsigned int m, std::string Link, 
-                     std::string Dist,
-                     unsigned int nthreads, double tol, int maxit){
+                  NumericVector init,
+                  std::string method,  unsigned int m, std::string Link, 
+                  std::string Dist,
+                  unsigned int nthreads, double tol, int maxit, bool GetInit){
   
-  arma::vec beta(x.cols(), arma::fill::zeros);
-  arma::mat Info(beta.n_elem, beta.n_elem);
+  // Initializing vectors and matrices
   const arma::mat X(x.begin(), x.rows(), x.cols(), false, true);
   const arma::vec Y(y.begin(), y.size(), false, true); 
   const arma::vec Offset(offset.begin(), offset.size(), false, true);
+  const arma::vec Init(init.begin(), init.size(), false, true);
+  arma::vec beta = Init;
+  arma::mat Info(beta.n_elem, beta.n_elem);
+  arma::vec SE1(beta.n_elem);
+  
+  // Initializing doubles
   double Iter;
+  double dispersion = 1;
 #ifdef _OPENMP
   omp_set_num_threads(nthreads);
 #endif
   
-  if(method == "BFGS"){
+  // Getting initial values
+  if(GetInit){
+    getInit(&beta, &X, &Y, &Offset, Dist, Link);
+  }
+  
+  // Fitting model
+  if(Dist == "gaussian" && Link == "identity"){
+    Iter = LinRegCpp(&beta, &X, &Y, &Offset, &SE1);
+  }else if(method == "BFGS"){
     Iter = BFGSGLMCpp(&beta, &X, &Y, &Offset, Link, Dist, tol, maxit);
   }
   else if(method == "LBFGS"){
@@ -557,43 +686,41 @@ List BranchGLMfit(NumericMatrix x, NumericVector y, NumericVector offset,
   else{
     Iter = FisherScoringGLMCpp(&beta, &X, &Y, &Offset, Link, Dist, tol, maxit);
   }
+  
+  // Checking for non-invertible fisher info error
   if(Iter == -2){
     stop("Algorithm failed to converge because the fisher info was not invertible");
   }
+  
+  // Calculating means
   arma::vec mu = LinkCpp(&X, &beta, &Offset, Link, Dist);
-  arma::vec Deriv = DerivativeCpp(&X, &beta, &Offset, &mu, Link, Dist);
-  arma::vec Var = Variance(&mu, Dist);
   
-  // Calculating information
-  
-  double dispersion = 1;
-  
-  if(Dist == "gaussian"){
-    dispersion = arma::accu(pow(Y - mu, 2)) / (X.n_rows);
-  }
-  Info = FisherInfoCpp(&X, &Deriv, &Var);
-  arma::mat InfoInv = Info;
-  if(!arma::inv_sympd(InfoInv, Info)){
-    stop("Fisher info not invertible");
-  }
-  // Calculating standard errors
+  // Calculating variances for betas for non-linear regression
+  if(Dist != "gaussian" || Link != "identity"){
     
-  const arma::vec SE1 = arma::diagvec(InfoInv);
+    // Calculating derivatives, and variances to be used for info
+    arma::vec Deriv = DerivativeCpp(&X, &beta, &Offset, &mu, Link, Dist);
+    arma::vec Var = Variance(&mu, Dist);
     
+    // Calculating info and initaliazing inverse info
+    Info = FisherInfoCpp(&X, &Deriv, &Var);
+    arma::mat InfoInv = Info;
+    
+    // Calculating inverse info and returning error if not invertible
+    if(!arma::inv_sympd(InfoInv, Info)){
+      stop("Fisher info not invertible");
+    }
+    
+    // Calculating variances which are later converted to SEs
+    SE1 = arma::diagvec(InfoInv);
+  }
+  
   NumericVector SE = NumericVector(SE1.begin(), SE1.end());
-    
+  
+  // Converting variances to SEs
   SE = sqrt(SE);
   
-  // Calculating z-values
-  
-  NumericVector z = NumericVector(beta.begin(), beta.end()) / SE;
-  
-  // Calculating p-values
-  
-  NumericVector p = 2 * pnorm(abs(z), 0, 1, false, false);
-  
   // Returning results
-  
   double satLogLik = LogLikelihoodSat(&X, &Y, Dist);
   double LogLik = -LogLikelihoodCpp(&X, &Y, &mu, Dist);
   double resDev = -2 * (LogLik - satLogLik);
@@ -601,10 +728,18 @@ List BranchGLMfit(NumericMatrix x, NumericVector y, NumericVector offset,
   
   NumericVector beta1 = NumericVector(beta.begin(), beta.end());
   
-  arma::vec linPreds = X * beta;
+  arma::vec linPreds = X * beta + Offset;
   
   NumericVector linPreds1 = NumericVector(linPreds.begin(), linPreds.end());
   
+  // Getting dispersion paramater
+  dispersion = GetDispersion(&X, &Y, &mu, LogLik, Dist, tol);
+  
+  // Checking for valid dispersion parameter
+  if(dispersion <= 0){
+    stop("dispersion parameter was estimated to be non-positive");
+  }
+  
   if(Dist == "gaussian"){
     double temp = Y.n_elem/2 * log(2*M_PI*dispersion);
     LogLik = LogLik / dispersion - temp;
@@ -613,8 +748,22 @@ List BranchGLMfit(NumericMatrix x, NumericVector y, NumericVector offset,
   else if(Dist == "poisson"){
     LogLik -=  LogFact(&Y);
     AIC = -2 * LogLik + 2 * (X.n_cols);
+  }else if(Dist == "gamma"){
+    double shape = 1 / dispersion;
+    LogLik = shape * LogLik + 
+      X.n_rows * (shape * log(shape) - lgamma(shape)) + 
+      (shape - 1) * arma::accu(log(Y));
+    AIC = -2 * LogLik + 2 * (X.n_cols + 1);
   }
   
+  SE = sqrt(dispersion) * SE;
+  
+  // Calculating z-values
+  NumericVector z = NumericVector(beta.begin(), beta.end()) / SE;
+  
+  // Calculating p-values
+  NumericVector p = 2 * pnorm(abs(z), 0, 1, false, false);
+  
 #ifdef _OPENMP
   omp_set_num_threads(1);
 #endif
@@ -632,20 +781,29 @@ List BranchGLMfit(NumericMatrix x, NumericVector y, NumericVector offset,
                             Named("linPreds") = linPreds1);
 }
 
+// Model fitting function for final model for variable selection functions
 List BranchGLMFitCpp(const arma::mat* X, const arma::vec* Y, const arma::vec* Offset,
-                std::string method,  unsigned int m, std::string Link, std::string Dist,
-                unsigned int nthreads, double tol, int maxit){
+                     std::string method,  unsigned int m, std::string Link, std::string Dist,
+                     unsigned int nthreads, double tol, int maxit, bool GetInit = true){
   
+  
+  // Initializing vectors and matrices
   arma::vec beta(X->n_cols, arma::fill::zeros);
   arma::mat Info(beta.n_elem, beta.n_elem);
+  arma::vec SE1(beta.n_elem);
+  // Initializing doubles
   double Iter;
-#ifdef _OPENMP
-  omp_set_num_threads(nthreads);
-#endif
+  double dispersion = 1;
+  // Getting initial values
+  if(GetInit){
+    getInit(&beta, X, Y, Offset, Dist, Link);
+  }
   
-  if(method == "BFGS"){
+  // Fitting model
+  if(Dist == "gaussian" && Link == "identity"){
+    Iter = LinRegCpp(&beta, X, Y, Offset, &SE1);
+  }else if(method == "BFGS"){
     Iter = BFGSGLMCpp(&beta, X, Y, Offset, Link, Dist, tol, maxit);
-    
   }
   else if(method == "LBFGS"){
     Iter = LBFGSGLMCpp(&beta, X, Y, Offset, Link, Dist, tol, maxit, m);
@@ -653,43 +811,41 @@ List BranchGLMFitCpp(const arma::mat* X, const arma::vec* Y, const arma::vec* Of
   else{
     Iter = FisherScoringGLMCpp(&beta, X, Y, Offset, Link, Dist, tol, maxit);
   }
+  
+  // Checking for non-invertible fisher info error
   if(Iter == -2){
-    stop("Algorithm failed because the fisher info was not invertible");
+    stop("Algorithm failed to converge because the fisher info was not invertible");
   }
+  
+  // Calculating means
   arma::vec mu = LinkCpp(X, &beta, Offset, Link, Dist);
-  arma::vec Deriv = DerivativeCpp(X, &beta, Offset, &mu, Link, Dist);
-  arma::vec Var = Variance(&mu, Dist);
   
-  // Calculating information
-  
-  double dispersion = 1;
-  
-  if(Dist == "gaussian"){
-    dispersion = arma::accu(pow(*Y - mu, 2)) / (X->n_rows);
+  // Calculating variances for betas for non-linear regression
+  if(Dist != "gaussian" || Link != "identity"){
+    
+    // Calculating derivatives, and variances to be used for info
+    arma::vec Deriv = DerivativeCpp(X, &beta, Offset, &mu, Link, Dist);
+    arma::vec Var = Variance(&mu, Dist);
+    
+    // Calculating info and initaliazing inverse info
+    Info = FisherInfoCpp(X, &Deriv, &Var);
+    arma::mat InfoInv = Info;
+    
+    // Calculating inverse info and returning error if not invertible
+    if(!arma::inv_sympd(InfoInv, Info)){
+      stop("Fisher info for final model not invertible");
+    }
+    
+    // Calculating variances which are later converted to SEs
+    SE1 = arma::diagvec(InfoInv);
   }
-  Info = FisherInfoCpp(X, &Deriv, &Var);
-  arma::mat InfoInv = Info;
-  if(!arma::inv_sympd(InfoInv, Info)){
-    stop("Fisher info not invertible");
-  }
-  // Calculating standard errors
-  
-  const arma::vec SE1 = arma::diagvec(InfoInv);
   
   NumericVector SE = NumericVector(SE1.begin(), SE1.end());
   
+  // Converting variances to SEs
   SE = sqrt(SE);
   
-  // Calculating z-values
-  
-  NumericVector z = NumericVector(beta.begin(), beta.end()) / SE;
-  
-  // Calculating p-values
-  
-  NumericVector p = 2 * pnorm(abs(z), 0, 1, false, false);
-  
   // Returning results
-  
   double satLogLik = LogLikelihoodSat(X, Y, Dist);
   double LogLik = -LogLikelihoodCpp(X, Y, &mu, Dist);
   double resDev = -2 * (LogLik - satLogLik);
@@ -697,10 +853,18 @@ List BranchGLMFitCpp(const arma::mat* X, const arma::vec* Y, const arma::vec* Of
   
   NumericVector beta1 = NumericVector(beta.begin(), beta.end());
   
-  arma::vec linPreds = *X * beta;
+  arma::vec linPreds = *X * beta + *Offset;
   
   NumericVector linPreds1 = NumericVector(linPreds.begin(), linPreds.end());
   
+  // Getting dispersion parameter
+  dispersion = GetDispersion(X, Y, &mu, LogLik, Dist, tol);
+  
+  // Checking for valid dispersion parameter
+  if(dispersion <= 0){
+    stop("dispersion parameter was estimated to be non-positive");
+  }
+  
   if(Dist == "gaussian"){
     double temp = Y->n_elem/2 * log(2*M_PI*dispersion);
     LogLik = LogLik / dispersion - temp;
@@ -709,12 +873,21 @@ List BranchGLMFitCpp(const arma::mat* X, const arma::vec* Y, const arma::vec* Of
   else if(Dist == "poisson"){
     LogLik -=  LogFact(Y);
     AIC = -2 * LogLik + 2 * (X->n_cols);
+  }else if(Dist == "gamma"){
+    double shape = 1 / dispersion;
+    LogLik = shape * LogLik + 
+      X->n_rows * (shape * log(shape) - lgamma(shape)) + 
+      (shape - 1) * arma::accu(log(*Y));
+    AIC = -2 * LogLik + 2 * (X->n_cols + 1);
   }
   
+  SE = sqrt(dispersion) * SE;
   
-#ifdef _OPENMP
-  omp_set_num_threads(1);
-#endif
+  // Calculating z-values
+  NumericVector z = NumericVector(beta.begin(), beta.end()) / SE;
+  
+  // Calculating p-values
+  NumericVector p = 2 * pnorm(abs(z), 0, 1, false, false);
   
   return List::create(Named("coefficients") = DataFrame::create(Named("Estimate") = beta1,  
                             Named("SE") = sqrt(dispersion) * SE,
